use proc_macro::TokenStream;
use proc_macro2::TokenStream as TokenStream2;
use quote::quote;
use update_macro_base::*;

fn resource_only_events_and_systems(update: &Update) -> TokenStream2 {
    let resource_tokens = update.resourece_tokens();

    let resource_types = &resource_tokens.resource_types;
    let resource_type_impls = &resource_tokens.resource_type_impls;
    let resources = &resource_tokens.resources;
    let resource_requirement = &resource_tokens.resource_requirement;
    let resource_store = &resource_tokens.resource_store;
    let resource_idents = &resource_tokens.resource_idents;

    quote! {}
}

fn events_and_systems(update: &Update) -> TokenStream2 {
    let query_tokens = update.query_tokens();

    let filter_types = &query_tokens.filter_types;
    let filter_type_impls = &query_tokens.filter_type_impls;
    let query_types = &query_tokens.query_types;
    let query_type_impls = &query_tokens.query_type_impls;
    let queries = query_tokens.queries;
    let filter_requirements = &query_tokens.filter_requirements;
    let component_requirements = &query_tokens.component_requirements;
    let query_count = query_tokens.query_count;
    let verify_dedup = &query_tokens.verify_dedup;
    let check_entities = &query_tokens.check_entities;
    let component_stores = &query_tokens.component_stores;
    let query_structs = &query_tokens.query_structs;
    let query_idents = &query_tokens.query_idents;

    let resource_tokens = update.resourece_tokens();

    let resource_types = &resource_tokens.resource_types;
    let resource_type_impls = &resource_tokens.resource_type_impls;
    let resources = &resource_tokens.resources;
    let resource_requirement = &resource_tokens.resource_requirement;
    let resource_store = &resource_tokens.resource_store;
    let resource_idents = &resource_tokens.resource_idents;

    quote! {
        impl<Func, #filter_type_impls #query_type_impls #resource_type_impls> SetCallback<#query_types, #resource_types, Func, #filter_types> for Button
        where
            Func: Fn(&mut Commands, &mut GuiHandler, #( #queries, )* #resources ) -> Result<()> + Send + Sync + 'static,
            #filter_requirements
            #component_requirements
            #resource_requirement
        {
            fn set_callback(&self, func: Func) {
                #( #verify_dedup )*

                self.click_executable.set_callback(
                    move |world: &mut World, commands: &mut Commands, gui_handler: &mut GuiHandler, _: ()| -> Result<()> {
                        let mut complying_entities: Vec<Vec<&EntityObject>> = vec![Vec::new(); #query_count];

                        for entity in unsafe { utilities::prelude::remove_life_time(world).entities() } {
                            let mut comply_queries = Vec::new();

                            #( #check_entities )*

                            if !comply_queries.is_empty() {
                                comply_queries
                                    .into_iter()
                                    .for_each(|q| complying_entities[q].push(entity));
                            }
                        }

                        // check if one of the queries could not be fullfilled
                        for queryable_entities in complying_entities.iter() {
                            if queryable_entities.is_empty() {
                                return Ok(());
                            }
                        }

                        #resource_store

                        for entities in complying_entities.into_iter().multi_cartesian_product() {
                            #( #component_stores )*
                            #( #query_structs )*

                            func(commands, gui_handler, #( #query_idents, )* #( #resource_idents, )*)?;
                        }

                        Ok(())
                    }
                )
            }
        }
    }
}

pub fn update(max_components: usize, max_resources: usize) -> TokenStream {
    let mut queries: Vec<Vec<usize>> = Vec::new();

    for c in Update::MIN_COMPONENTS..=max_components {
        queries.push(vec![c]);
    }

    // for x in Update::MIN_COMPONENTS..=max_components {
    //     for y in Update::MIN_COMPONENTS..=max_components {
    //         queries.push(vec![x, y]);
    //     }
    // }

    // for x in Update::MIN_COMPONENTS..=max_components {
    //     for y in Update::MIN_COMPONENTS..=max_components {
    //         for z in Update::MIN_COMPONENTS..=max_components {
    //             queries.push(vec![x, y, z]);
    //         }
    //     }
    // }

    let mut resources = Vec::new();

    for resource in Update::MIN_RESOURCE..=max_resources {
        resources.push(resource);
    }

    let mut updates = Vec::new();

    for query in queries.iter() {
        for resource_count in resources.iter() {
            updates.push(events_and_systems(&Update::new(
                query.iter().cloned(),
                *resource_count,
            )));
        }
    }

    let mut events = Vec::new();

    for resource_count in resources.iter() {
        let q =
            resource_only_events_and_systems(&Update::new(vec![1].into_iter(), *resource_count));

        events.push(q);
    }

    // updates.push(events_and_systems(&Update::new(vec![2].into_iter(), 0)));

    // let q = quote! {
    //     #( #updates )*
    // };

    let q = quote! {
        #( #updates )*
        #( #events )*
    };

    panic!("{q}");

    TokenStream::from(q)
}
